//
//  XSPEC12  November 2003
//
//
#include <XSUser/Global/XSGlobal.h>
#include <XSUser/Global/Global.h>
#include <XSUtil/Parse/XSparse.h>
#include <unistd.h> 
#include <XSstreams.h>
#include <XSContainer.h>
#include <xsTypes.h>
#include <cstring>


int 
XSGlobal::xsLmod(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
        static const string PKGINDEX("pkg_mkIndex ");
        static const string LAPPEND("lappend ");
        static const string AUTOPATH("auto_path ");
        string cmd = string(static_cast<char*>(cdata));
        if ( objc == 1 )
        {
                XSGlobal::printDocs(static_cast<char*>(cdata),"?");      
        }
        else
        {
           string arg(Tcl_GetString(objv[1]));
           if ( arg[0] == '?') 
           {
                   XSGlobal::printDocs(static_cast<char*>(cdata),"?");   
                   return globalData->autoSave(TCL_OK);
           }
           else  
           {
#ifdef __CYGWIN__
   tcerr << "\n***Error: Lmod command can only handle dynamically built local model\n"
         << "          libraries, and therefore cannot be run on CYGWIN.\n"
         << "          See the initpackage help to use the static_initpackage command"
         << "          for building and linking static local model libraries." << std::endl;
   return globalData->autoSave(TCL_ERROR);
#endif
              // simple case: use default local model library, and 
              // the model description has filename packageName.dat
              char* pkgName = Tcl_GetString(objv[1]);
              if ( objc == 2 )
              {
                 if ( globalData->defaultLocalModelDirectory().length())
                 {
                    string sCmd = LAPPEND;
                    sCmd += AUTOPATH;
                    sCmd += globalData->defaultLocalModelDirectory();
                    // To avoid passing const char* to Tcl_Eval:
                    char* c_sCmd(new char[sCmd.length() + 1]);
                    sCmd.copy(c_sCmd, string::npos);
                    c_sCmd[sCmd.length()] = 0;
                    Tcl_Eval(tclInterp,c_sCmd);
                    delete [] c_sCmd;
                    char* pkgCmd(new char[PKGINDEX.length()+ 
                        globalData->defaultLocalModelDirectory().length()+1]);
                    strcpy(pkgCmd,PKGINDEX.c_str());
                    strcat(pkgCmd,
                            globalData->defaultLocalModelDirectory().c_str());
                    Tcl_Eval(tclInterp,pkgCmd);
                    delete [] pkgCmd;
                    const char* pkgTest =
                                    Tcl_PkgRequire(tclInterp,pkgName,NULL,0);
                    Tcl_ResetResult(tclInterp);
                    if (!pkgTest)
                    {
                       tcerr << "\n***Error: Xspec was unable to load the model package: "
                             << pkgName << "\n   Either it could not find the model library file "
                             <<"in the directory:\n"<<globalData->defaultLocalModelDirectory()
                             <<"\n   or the file contains errors."
                             <<"\n   (try \"load (path)/(lib filename)\" for more error info)"
                             <<std::endl;
                       return globalData->autoSave(TCL_ERROR);
                             
                    }
                    else
                    {
                       tcout << "Model package " << pkgName << " successfully loaded."
                             << std::endl;
                    }
                    return globalData->autoSave(TCL_OK);  
                 }  
                 else
                 {
                         tcerr << " No valid setting for the default local model "
                               << " directory. \nSet a path for the model library or "
                               << " modify the default setting in Xspec.init\n";
                         return globalData->autoSave(TCL_ERROR);
                 }  
              }
              else if (objc >= 3 )
              {
                 string directory =  string(Tcl_GetString(objv[2]));
                 string fullDirectory("");
                 if ( !strchr("/$~.",directory[0]) )
                 {
                         tcerr << "*** Error: local model directory must either be"
                               << " an absolute path \n*** or begin with a defined "
                               << " environment variable, or shell character "
                               << " {~,./,../}\n";
                         return globalData->autoSave(TCL_ERROR);
                 } 
                 else
                 {
                    if (XSparse::addToLibraryPath(directory,fullDirectory,
                                                    F_OK|R_OK|W_OK|X_OK))
                    {
                       string sCmd = LAPPEND;
                       sCmd += AUTOPATH;
                       sCmd += fullDirectory;
                       char* c_sCmd(new char[sCmd.length() + 1]);
                       sCmd.copy(c_sCmd, string::npos);
                       c_sCmd[sCmd.length()] = 0;
                       Tcl_Eval(tclInterp,c_sCmd);
                       delete [] c_sCmd;
                       
                       // nothing in here can throw.
                       char* pkgCmd(new char[PKGINDEX.length()
                                       +fullDirectory.length()+1]);
                       strcpy(pkgCmd,PKGINDEX.c_str());
                       strcat(pkgCmd,fullDirectory.c_str());
                       Tcl_Eval(tclInterp,pkgCmd);
                       delete [] pkgCmd;  
                       const char* pkgTest 
                               = Tcl_PkgRequire(tclInterp,pkgName,NULL,0);
                       Tcl_ResetResult(tclInterp);
                       if (!pkgTest)
                       {
                          tcerr << "\n***Error: Xspec was unable to load the model package: "
                                << pkgName << "\n   Either it could not find the model library file "
                                <<"in the directory:\n"<<fullDirectory
                                <<"\n   or the file contains errors."
                                <<"\n   (try \"load (path)/(lib filename)\" for more error info)"
                                <<std::endl;
                          return globalData->autoSave(TCL_ERROR);

                       }
                       else
                       {
                          tcout << "Model package " << pkgName << " successfully loaded."
                                << std::endl;
                       }
                    }
                    else
                    {
                            tcerr << "*** Error: local model directory \n"
                               << "*** " << fullDirectory 
                               << "\n*** cannot be added to the load path. "
                               << "\n*** most likely it has an undefined "
                               << "environment variable "
                               << "\n*** or insufficient access permissions "
                               << "(The directory must exist and be writable)"
                               << "\n";      
                            return globalData->autoSave(TCL_ERROR); 
                    } 

                 }      
              }
           }
        }
        
        
        
        return globalData->autoSave(TCL_OK);
        
}
